package com.ym.mec.web.controller;

import com.alibaba.fastjson.JSON;
import com.ym.mec.auth.api.client.SysUserFeignService;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.dao.AppVersionInfoDao;
import com.ym.mec.biz.dal.dto.SysMessageDto;
import com.ym.mec.biz.dal.enums.MessageSendMode;
import com.ym.mec.biz.dal.page.SysMessageQueryInfo;
import com.ym.mec.biz.service.SysMessageService;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.page.PageInfo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.*;

@RestController
@Api(tags = "消息服务")
@RequestMapping("${app-config.url.web:}/schoolMessage")
public class SchoolMessageController extends BaseController {

    @Autowired
    private SysMessageService sysMessageService;

    @Autowired
    private SysUserFeignService sysUserFeignService;

    @Autowired
    private AppVersionInfoDao appVersionInfoDao;

    @ApiOperation("获取所有消息列表")
    @PostMapping(value = "list")
    public Object list(@RequestBody SysMessageQueryInfo queryInfo){
        SysUser sysUser = sysUserFeignService.queryUserInfo();
        if (sysUser == null) {
            return failed(HttpStatus.FORBIDDEN, "请登录");
        }

        queryInfo.setUserId(sysUserFeignService.queryUserInfo().getId());
        queryInfo.setType(3);
        queryInfo.setJpushType("SCHOOL");
        PageInfo<SysMessageDto> pageInfo = sysMessageService.queryListPage(queryInfo);
        return succeed(pageInfo);
    }

    @ApiOperation("获取消息分类类型")
    @GetMapping(value = "typeList")
    public Object typeList(String memo){

        return succeed(JSON.parseArray("[{\"value\":\"全部消息\",\"key\":\"ALL\"}," +
                "{\"value\":\"老师信息\",\"key\":\"TEACHER\"}," +
                "{\"value\":\"学员信息\",\"key\":\"STUDENT\"}," +
                "{\"value\":\"系统消息\",\"key\":\"SYSTEM\"},"));
    }

    @ApiOperation("一键已读")
    @PostMapping("batchSetRead")
    public Object batchSetRead() {
        int status = 1;
        SysUser sysUser = sysUserFeignService.queryUserInfo();
        if (sysUser == null) {
            return failed(HttpStatus.FORBIDDEN, "请登录");
        }
        return sysMessageService.updateStatus(sysUser.getId(), status,"SCHOOL") > 0 ? succeed() : failed();
    }
    @ApiOperation("设置已读")
    @PostMapping("setRead")
    public Object setRead(@ApiParam("id") Long id) {
        int status = 1;
        SysUser sysUser = sysUserFeignService.queryUserInfo();
        if (sysUser == null) {
            return failed(HttpStatus.FORBIDDEN, "请登录");
        }
        return sysMessageService.updateOneStatus(id, status) > 0 ? succeed() : failed();
    }

    @ApiOperation(value = "查询用户未读消息条数")
    @GetMapping("/queryCountOfUnread")
    public Object queryCountOfUnread() {
        SysUser sysUser = sysUserFeignService.queryUserInfo();
        if (sysUser == null) {
            return failed(HttpStatus.FORBIDDEN, "请登录");
        }
        return succeed(sysMessageService.queryCountOfUnread(MessageSendMode.PUSH, sysUser.getId(),"SCHOOL", null));
    }
}
